import java.util.*;
import java.io.*;

/**
   Implementazione di un possibile <i>Storage</i> su file.
   Lo <i>State</i> trattato DEVE essere SERIALIZZABILE!

   @see SerialStateI
   @author Andrea Trentini
*/
public class SerialStorage implements StorageI
{
   /** Output */
   protected ObjectOutputStream so;

   /** Input */
   protected ObjectInputStream si;

   /** L'ultimo <i>State</i> che e' passato di qui... */
   protected StateI lastState;

   /** Directory associata */
   protected File rootDir;

   /**
      Inizializza un nuovo <i>Storage</i>,
      bisogna dirgli in che directory mettere
      la roba
      @param rDir directory da usare
   */
   public SerialStorage(String rDir)
      throws FileNotFoundException, IOException
   {
      rootDir = new File(rDir);

      if(!rootDir.exists() || !rootDir.isDirectory())
         throw new FileNotFoundException("La directory "+rootDir+" non esiste!!!");

      if(!rootDir.canRead() || !rootDir.canWrite())
         throw new IOException("La directory "+rootDir+" non e' accessibile!!!");
   }

   /**
      aggiungere dettagli?
   */
   public String toString()
   {
      return
         Html.OBEGIN+
         Html.instance(super.toString())+
         Html.field("path",rootDir.getAbsolutePath())+
         Html.field("keys",keys().toString())+
         Html.field("lastState",""+lastState)+
         Html.OEND;
   }

   /** Accetta SOLO oggetti serializzabili. */
   public boolean saveState(StateI s)
   {
      if(s instanceof SerialStateI)
      {
         try
         {
            so = new ObjectOutputStream(
                  new FileOutputStream(
                     new File(rootDir, s.getId())));
            
            so.writeObject(lastState=s);
      
            so.flush();
            so.close();
         }
         catch(IOException e)
         {
            Debug.px(e);
            return false;
         }
   
         return true;
      }
      else
         return false;
   }

   public StateI retrieveState(String id)
   {
      try
      {
         si = new ObjectInputStream(
               new FileInputStream(
                  new File(rootDir, id)));
         
         lastState = (StateI) si.readObject();

         si.close();

         return lastState;
      }
      catch(Exception e)
      {Debug.px(e);}

      return null;
   }

   public int count()
   {return rootDir.list().length;}

   /**
      [TODO] Ritorna quanti state ci sono, filtrato.
   */
   public Enumeration keys(String filter)
   {
      return null;
   }

   /**
      Ritorna quanti state ci sono.
   */
   public Enumeration keys()
   {
      // NON va bene!!!, il VectorEnumerator NON e' serializzabile!!!
      // return vnames().elements();

      return new ObjectArrayEnumerator(rootDir.list());
   }

   /**
      Per sapere se c'e' uno stato.
   */
   public boolean isIn(String id)
   {return (new File(rootDir,id)).exists();}

   /**
      E' autoesplicativo? ;-)
   */
   public boolean deleteState(String id)
   {return (new File(rootDir,id)).delete();}
}
